package scene;

import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Parser {
    
    public final char zero = 0;
    public final char quote1 = '\'';
    public final char quote2 = '\"';

    public enum TokenKind { identifier, number, string, separator, eos };
    
    public TokenKind kind;
    public String token;

    private BufferedReader f = null;
    private char ch;
    private int line;
    private int col;
    
    Parser (String fileName)
    {
        f = null;
        try {
            FileReader r = new FileReader (fileName);
            f = new BufferedReader (r);
        } 
        catch (FileNotFoundException ex) { error (ex.toString()); }
        // catch (IOException ex) { error (ex.toString()); }
        
        kind = TokenKind.eos;
        token = "";
        
        ch = zero;
        line = 1;
        col = 0;
        nextChar ();
        nextToken ();
    }
    
    public void error (String s)
    {
       s = "line " + line + ": column " + col + ": " + s;
       System.out.println(s);
       throw new RuntimeException (s);
    }

    public void error (Exception e)
    {
        error (e.toString());
    }
    
    private void nextChar () 
    {
        ch = zero;
        if (f != null)
            try 
            {
                int n  = f.read();
                if (n < 0)
                    ch = zero;
                else
                    ch = (char) n;

                if (ch == '\n') 
                {
                   line ++;
                   col = 0;
                }
                else if (ch != '\r')
                {
                   col ++;
                }
            } 
            catch (IOException ex) { error (ex.toString()); }
    }
    
    public void nextToken () 
    {
        token = "";
        
        while (ch != zero && ch <= ' ')
            nextChar();
        
        if ( Character.isAlphabetic(ch) )
        {
            kind = TokenKind.identifier;
            while ( Character.isAlphabetic(ch) || Character.isDigit(ch) || ch == '_' )
            {
               token = token + ch;
               nextChar();
            }
        }
        else if ( Character.isDigit(ch) )
        {
            kind = TokenKind.number;
            while ( Character.isDigit (ch) )
            {
               token = token + ch;
               nextChar();
            }
        }
        else if ( ch == quote2 )
        {
            kind = TokenKind.string;
            nextChar(); // skip quote
            while ( ch != quote2 )
            {
               token = token + ch;
               nextChar();
            }
            nextChar(); // skip quote
        }
        else if (ch != zero)
        {
            kind = TokenKind.separator;
            token = "" + ch;
            nextChar();
        }
        else
        {
            kind = TokenKind.eos;
        }
    }
    
    public String readIdentifier ()
    {
        if (kind != TokenKind.identifier)
            error ("Identifier expected");
        String s = token;
        nextToken ();
        return s;
    }

    public boolean isSeparator (char c)
    {
        return kind == TokenKind.separator && token.equals (""+c) ;
    }
    
    public void checkSeparator (char c)
    {
        if (! isSeparator(c))
            error (c + " expected");
        nextToken ();
    }
    
    public Object readValue ()
    {
        Object result = null;

        if (kind == TokenKind.number)
        {
            result = Integer.getInteger(token);
            nextToken ();
        }
        else if (kind == TokenKind.string)
        {
            result = token;
            nextToken ();
        }
        else
            error ("String or Number expected");
            
        return result;   
    }
}
